package pwc.taxtech.atms.controller;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.organization.dpo.OrganizationDirectorDto;
import pwc.taxtech.atms.organization.dpo.OrganizationHKDto;
import pwc.taxtech.atms.organization.dpo.OrganizationShareholderDto;
import pwc.taxtech.atms.organization.entity.OrganizationDirector;
import pwc.taxtech.atms.organization.entity.OrganizationShareholder;
import pwc.taxtech.atms.service.impl.OrganizationHKServiceImpl;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping("/api/v1/orgHK/")
public class OrganizationHKController {

    private static Logger logger = LoggerFactory.getLogger(OrganizationHKController.class);

    @Autowired
    OrganizationHKServiceImpl organizationHKService;

    @RequestMapping(value = "getOrgInfoList", method = RequestMethod.GET)
    public @ResponseBody
    List<OrganizationHKDto> getOrgInfoList() {
        logger.info("GET /api/v1/orgHK/getOrgInfoList");
        return organizationHKService.getOrgInfo();
    }

    @RequestMapping(value = "enableOrgs", method = RequestMethod.GET)
    public @ResponseBody
    OperationResultDto enableOrgs(@RequestParam("orgId") Long orgId) {
        logger.info("POST /api/v1/orgHK/enableOrgs");
        return organizationHKService.enableOrgs(orgId);
    }

    @RequestMapping(value = "disableOrgs", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto disableOrgs(@RequestBody List<Long> orgIds) {
        logger.info("POST /api/v1/orgHK/disableOrgs");
        return organizationHKService.disableOrgs(orgIds);
    }

    @RequestMapping(value = "update", method = RequestMethod.PUT)
    public @ResponseBody
    OperationResultDto<Object> updateOrg(@RequestBody OrganizationHKDto orgDto) {
        logger.info("PUT /api/v1/orgHK/update");
        return organizationHKService.updateOrg(orgDto);
    }

    @RequestMapping(value = "getjson", method = RequestMethod.GET)
    public @ResponseBody
    List<NavTreeDto> getOrgListToJson(@RequestParam("useType") Integer useType) {
        logger.info("GET /api/v1/orgHK/getjson");
        return organizationHKService.getOrgListToJson(useType);
    }

    @RequestMapping(value = "displaySingle", method = RequestMethod.GET)
    public @ResponseBody
    OrganizationHKDto getSingleOrgByOrgId(@RequestParam("orgId") Long orgId) {
        logger.info("POST /api/v1/orgHK/displaySingle");
        return organizationHKService.getSingleOrgByOrgId(orgId);
    }

    @RequestMapping(value = "add", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> addOrg(@RequestBody OrganizationHKDto orgDto) {
        logger.info("POST /api/v1/orgHK/add");
        return organizationHKService.addOrg(orgDto);
    }

    @RequestMapping(value = "addDirecotr", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> addDirector(@RequestBody OrganizationDirector organizationDirector) {
        logger.info("POST /api/v1/orgHK/addDirecotr");
        return organizationHKService.addOrgDirector(organizationDirector);
    }

    @RequestMapping(value = "addShareholder", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> addShareholder(@RequestBody OrganizationShareholder organizationShareholder) {
        logger.info("POST /api/v1/orgHK/addShareholder");
        return organizationHKService.addOrgShareholder(organizationShareholder);
    }

    @RequestMapping(value = "updateDirector", method = RequestMethod.PUT)
    public @ResponseBody
    OperationResultDto<Object> updateDirector(@RequestBody OrganizationDirector organizationDirector) {
        logger.info("PUT /api/v1/orgHK/updateDirector");
        return organizationHKService.updateDirector(organizationDirector);
    }

    @RequestMapping(value = "updateShareholder", method = RequestMethod.PUT)
    public @ResponseBody
    OperationResultDto<Object> updateShareholder(@RequestBody OrganizationShareholder organizationShareholder) {
        logger.info("PUT /api/v1/orgHK/updateShareholder");
        return organizationHKService.updateShareholder(organizationShareholder);
    }

    @RequestMapping(value = "updateDirectors", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> updateDirectors(@RequestBody List<OrganizationDirectorDto> organizationDirectorDtos) {
        logger.info("POST /api/v1/orgHK/updateDirectors");
        return organizationHKService.updateDirectors(organizationDirectorDtos);
    }

    @RequestMapping(value = "updateShareholders", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> updateShareholders(@RequestBody List<OrganizationShareholderDto> organizationShareholderDtos) {
        logger.info("POST /api/v1/orgHK/updateShareholders");
        return organizationHKService.updateShareholders(organizationShareholderDtos);
    }

    @RequestMapping(value = "upload", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> uploadOrgAttach(@RequestParam(value = "file") CommonsMultipartFile[] files, @RequestParam String entityId) {
        logger.info("POST /api/v1/orgHK/upload");
        logger.debug("enter upload file method");
        OperationResultDto<Object> resultDto = new OperationResultDto<>();
        logger.debug("validate parameters");
        if (files == null || files.length == 0) {
            resultDto.setResult(false);
            resultDto.setResultMsg(ErrorMessage.NoFile);
            return resultDto;
        }

        for (CommonsMultipartFile file : files) {
            if (file == null || file.getSize() <= 0) {
                resultDto.setResult(false);
                resultDto.setResultMsg(ErrorMessage.NoFile);
                return resultDto;
            }
        }
        logger.debug("validate entityId");
        if (StringUtils.isEmpty(entityId) || !organizationHKService.checkOrgExist(entityId)) {
            resultDto.setResult(false);
            logger.warn("organization [{}] not exist", entityId);
            resultDto.setResultMsg("organization " + entityId + " not exist");
            return resultDto;
        }

        logger.debug("start upload");
        try {
            resultDto = organizationHKService.uploadAttachs(files, entityId);
        } catch (ApplicationException ex) {
            resultDto.setResult(false);
        }
        return resultDto;

    }

    @RequestMapping(value = "downloadAttachment", method = RequestMethod.GET)
    public @ResponseBody
    void downloadAttachment(HttpServletRequest request, HttpServletResponse response, @RequestParam("attachmentId") Long attachmentId) {
        logger.info("POST /api/v1/orgHK/downloadAttachment");
        organizationHKService.downloadAttachment(request, response, attachmentId);
    }

    @RequestMapping(value = "deleteAttachment", method = RequestMethod.DELETE)
    public @ResponseBody
    OperationResultDto<Object> deleteAttachment(@RequestParam("attachmentId") Long attachmentId) {
        logger.info("POST /api/v1/orgHK/deleteAttachment");
        return organizationHKService.deleteAttachment(attachmentId);
    }

}
